@@ -16,13 +16,17 @@
 # specific language governing permissions and limitations
 # under the License.
 """DAG run APIs."""
+from __future__ import annotations
+
 from datetime import datetime
-from typing import Dict
+
+from deprecated import deprecated
 
 from airflow.api.common.experimental import check_and_get_dag, check_and_get_dagrun
 
 
-def get_dag_run_state(dag_id: str, execution_date: datetime) -> Dict[str, str]:
+@deprecated(reason="Use DagRun().get_state() instead", version="2.2.4")
+def get_dag_run_state(dag_id: str, execution_date: datetime) -> dict[str, str]:
     """Return the Dag Run state identified by the given dag_id and execution_date.
 
     :param dag_id: DAG id
