@@ -14,7 +14,9 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from typing import NamedTuple, Optional
+from __future__ import annotations
+
+from typing import NamedTuple
 
 from marshmallow import Schema, fields
 
@@ -30,7 +32,7 @@ class LogResponseObject(NamedTuple):
     """Log Response Object"""
 
     content: str
-    continuation_token: Optional[str]
+    continuation_token: str | None
 
 
 logs_schema = LogsSchema()
