@@ -16,31 +16,38 @@
 # specific language governing permissions and limitations
 # under the License.
 """Task sub-commands"""
+from __future__ import annotations
+
+import datetime
 import importlib
 import json
 import logging
 import os
 import textwrap
 from contextlib import contextmanager, redirect_stderr, redirect_stdout, suppress
-from typing import List, Optional
+from typing import Generator, Union
 
 from pendulum.parsing.exceptions import ParserError
 from sqlalchemy.orm.exc import NoResultFound
+from sqlalchemy.orm.session import Session
 
 from airflow import settings
 from airflow.cli.simple_table import AirflowConsole
 from airflow.configuration import conf
-from airflow.exceptions import AirflowException, DagRunNotFound
+from airflow.exceptions import AirflowException, DagRunNotFound, TaskInstanceNotFound
 from airflow.executors.executor_loader import ExecutorLoader
 from airflow.jobs.local_task_job import LocalTaskJob
 from airflow.models import DagPickle, TaskInstance
+from airflow.models.baseoperator import BaseOperator
 from airflow.models.dag import DAG
 from airflow.models.dagrun import DagRun
 from airflow.ti_deps.dep_context import DepContext
 from airflow.ti_deps.dependencies_deps import SCHEDULER_QUEUED_DEPS
+from airflow.typing_compat import Literal
 from airflow.utils import cli as cli_utils
 from airflow.utils.cli import (
     get_dag,
+    get_dag_by_deserialization,
     get_dag_by_file_location,
     get_dag_by_pickle,
     get_dags,
@@ -48,49 +55,127 @@ from airflow.utils.cli import (
 )
 from airflow.utils.dates import timezone
 from airflow.utils.log.logging_mixin import StreamLogWriter
+from airflow.utils.log.secrets_masker import RedactedIO
 from airflow.utils.net import get_hostname
-from airflow.utils.session import create_session, provide_session
+from airflow.utils.session import NEW_SESSION, create_session, provide_session
+from airflow.utils.state import DagRunState
+
+log = logging.getLogger(__name__)
+
+CreateIfNecessary = Union[Literal[False], Literal["db"], Literal["memory"]]
 
 
-def _get_dag_run(dag, exec_date_or_run_id, create_if_necssary, session):
-    dag_run = dag.get_dagrun(run_id=exec_date_or_run_id, session=session)
-    if dag_run:
-        return dag_run
+def _generate_temporary_run_id() -> str:
+    """Generate a ``run_id`` for a DAG run that will be created temporarily.
 
-    execution_date = None
-    with suppress(ParserError, TypeError):
-        execution_date = timezone.parse(exec_date_or_run_id)
+    This is used mostly by ``airflow task test`` to create a DAG run that will
+    be deleted after the task is run.
+    """
+    return f"__airflow_temporary_run_{timezone.utcnow().isoformat()}__"
 
-    if create_if_necssary and not execution_date:
-        return DagRun(dag_id=dag.dag_id, run_id=exec_date_or_run_id)
-    try:
-        return (
-            session.query(DagRun)
-            .filter(
-                DagRun.dag_id == dag.dag_id,
-                DagRun.execution_date == execution_date,
+
+def _get_dag_run(
+    *,
+    dag: DAG,
+    create_if_necessary: CreateIfNecessary,
+    exec_date_or_run_id: str | None = None,
+    session: Session,
+) -> tuple[DagRun, bool]:
+    """Try to retrieve a DAG run from a string representing either a run ID or logical date.
+
+    This checks DAG runs like this:
+
+    1. If the input ``exec_date_or_run_id`` matches a DAG run ID, return the run.
+    2. Try to parse the input as a date. If that works, and the resulting
+       date matches a DAG run's logical date, return the run.
+    3. If ``create_if_necessary`` is *False* and the input works for neither of
+       the above, raise ``DagRunNotFound``.
+    4. Try to create a new DAG run. If the input looks like a date, use it as
+       the logical date; otherwise use it as a run ID and set the logical date
+       to the current time.
+    """
+    if not exec_date_or_run_id and not create_if_necessary:
+        raise ValueError("Must provide `exec_date_or_run_id` if not `create_if_necessary`.")
+    execution_date: datetime.datetime | None = None
+    if exec_date_or_run_id:
+        dag_run = dag.get_dagrun(run_id=exec_date_or_run_id, session=session)
+        if dag_run:
+            return dag_run, False
+        with suppress(ParserError, TypeError):
+            execution_date = timezone.parse(exec_date_or_run_id)
+        try:
+            dag_run = (
+                session.query(DagRun)
+                .filter(DagRun.dag_id == dag.dag_id, DagRun.execution_date == execution_date)
+                .one()
             )
-            .one()
+        except NoResultFound:
+            if not create_if_necessary:
+                raise DagRunNotFound(
+                    f"DagRun for {dag.dag_id} with run_id or execution_date "
+                    f"of {exec_date_or_run_id!r} not found"
+                ) from None
+        else:
+            return dag_run, False
+
+    if execution_date is not None:
+        dag_run_execution_date = execution_date
+    else:
+        dag_run_execution_date = timezone.utcnow()
+
+    if create_if_necessary == "memory":
+        dag_run = DagRun(dag.dag_id, run_id=exec_date_or_run_id, execution_date=dag_run_execution_date)
+        return dag_run, True
+    elif create_if_necessary == "db":
+        dag_run = dag.create_dagrun(
+            state=DagRunState.QUEUED,
+            execution_date=dag_run_execution_date,
+            run_id=_generate_temporary_run_id(),
+            session=session,
         )
-    except NoResultFound:
-        if create_if_necssary:
-            return DagRun(dag.dag_id, execution_date=execution_date)
-        raise DagRunNotFound(
-            f"DagRun for {dag.dag_id} with run_id or execution_date of {exec_date_or_run_id!r} not found"
-        ) from None
+        return dag_run, True
+    raise ValueError(f"unknown create_if_necessary value: {create_if_necessary!r}")
 
 
 @provide_session
-def _get_ti(task, exec_date_or_run_id, create_if_necssary=False, session=None):
+def _get_ti(
+    task: BaseOperator,
+    map_index: int,
+    *,
+    exec_date_or_run_id: str | None = None,
+    pool: str | None = None,
+    create_if_necessary: CreateIfNecessary = False,
+    session: Session = NEW_SESSION,
+) -> tuple[TaskInstance, bool]:
     """Get the task instance through DagRun.run_id, if that fails, get the TI the old way"""
-    dag_run = _get_dag_run(task.dag, exec_date_or_run_id, create_if_necssary, session)
+    if not exec_date_or_run_id and not create_if_necessary:
+        raise ValueError("Must provide `exec_date_or_run_id` if not `create_if_necessary`.")
+    if task.is_mapped:
+        if map_index < 0:
+            raise RuntimeError("No map_index passed to mapped task")
+    elif map_index >= 0:
+        raise RuntimeError("map_index passed to non-mapped task")
+    dag_run, dr_created = _get_dag_run(
+        dag=task.dag,
+        exec_date_or_run_id=exec_date_or_run_id,
+        create_if_necessary=create_if_necessary,
+        session=session,
+    )
 
-    ti = dag_run.get_task_instance(task.task_id)
-    if not ti and create_if_necssary:
-        ti = TaskInstance(task, run_id=None)
+    ti_or_none = dag_run.get_task_instance(task.task_id, map_index=map_index, session=session)
+    if ti_or_none is None:
+        if not create_if_necessary:
+            raise TaskInstanceNotFound(
+                f"TaskInstance for {task.dag.dag_id}, {task.task_id}, map={map_index} with "
+                f"run_id or execution_date of {exec_date_or_run_id!r} not found"
+            )
+        # TODO: Validate map_index is in range?
+        ti = TaskInstance(task, run_id=dag_run.run_id, map_index=map_index)
         ti.dag_run = dag_run
-    ti.refresh_from_task(task)
-    return ti
+    else:
+        ti = ti_or_none
+    ti.refresh_from_task(task, pool_override=pool)
+    return ti, dr_created
 
 
 def _run_task_by_selected_method(args, dag: DAG, ti: TaskInstance) -> None:
@@ -181,18 +266,17 @@ def _run_raw_task(args, ti: TaskInstance) -> None:
         mark_success=args.mark_success,
         job_id=args.job_id,
         pool=args.pool,
-        error_file=args.error_file,
     )
 
 
-def _extract_external_executor_id(args) -> Optional[str]:
+def _extract_external_executor_id(args) -> str | None:
     if hasattr(args, "external_executor_id"):
         return getattr(args, "external_executor_id")
     return os.environ.get("external_executor_id", None)
 
 
 @contextmanager
-def _capture_task_logs(ti):
+def _capture_task_logs(ti: TaskInstance) -> Generator[None, None, None]:
     """Manage logging context for a task run
 
     - Replace the root logger configuration with the airflow.task configuration
@@ -226,7 +310,7 @@ def _capture_task_logs(ti):
             root_logger.handlers[:] = orig_handlers
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 def task_run(args, dag=None):
     """Run a single task instance.
 
@@ -247,13 +331,14 @@ def task_run(args, dag=None):
         unsupported_options = [o for o in RAW_TASK_UNSUPPORTED_OPTION if getattr(args, o)]
 
         if unsupported_options:
+            unsupported_raw_task_flags = ', '.join(f'--{o}' for o in RAW_TASK_UNSUPPORTED_OPTION)
+            unsupported_flags = ', '.join(f'--{o}' for o in unsupported_options)
             raise AirflowException(
-                "Option --raw does not work with some of the other options on this command. You "
-                "can't use --raw option and the following options: {}. You provided the option {}. "
-                "Delete it to execute the command".format(
-                    ", ".join(f"--{o}" for o in RAW_TASK_UNSUPPORTED_OPTION),
-                    ", ".join(f"--{o}" for o in unsupported_options),
-                )
+                "Option --raw does not work with some of the other options on this command. "
+                "You can't use --raw option and the following options: "
+                f"{unsupported_raw_task_flags}. "
+                f"You provided the option {unsupported_flags}. "
+                "Delete it to execute the command."
             )
     if dag and args.pickle:
         raise AirflowException("You cannot use the --pickle option when using DAG.cli() method.")
@@ -269,27 +354,34 @@ def task_run(args, dag=None):
 
     settings.MASK_SECRETS_IN_LOGS = True
 
-    # IMPORTANT, have to use the NullPool, otherwise, each "run" command may leave
+    # IMPORTANT, have to re-configure ORM with the NullPool, otherwise, each "run" command may leave
     # behind multiple open sleeping connections while heartbeating, which could
     # easily exceed the database connection limit when
     # processing hundreds of simultaneous tasks.
-    settings.configure_orm(disable_connection_pool=True)
+    settings.reconfigure_orm(disable_connection_pool=True)
 
     if args.pickle:
         print(f'Loading pickle id: {args.pickle}')
         dag = get_dag_by_pickle(args.pickle)
     elif not dag:
-        dag = get_dag(args.subdir, args.dag_id)
+        if args.local:
+            try:
+                dag = get_dag_by_deserialization(args.dag_id)
+            except AirflowException:
+                print(f'DAG {args.dag_id} does not exist in the database, trying to parse the dag_file')
+                dag = get_dag(args.subdir, args.dag_id)
+        else:
+            dag = get_dag(args.subdir, args.dag_id)
     else:
         # Use DAG from parameter
         pass
     task = dag.get_task(task_id=args.task_id)
-    ti = _get_ti(task, args.execution_date_or_run_id)
+    ti, _ = _get_ti(task, args.map_index, exec_date_or_run_id=args.execution_date_or_run_id, pool=args.pool)
     ti.init_run_context(raw=args.raw)
 
     hostname = get_hostname()
 
-    print(f"Running {ti} on host {hostname}")
+    log.info("Running %s on host %s", ti, hostname)
 
     if args.interactive:
         _run_task_by_selected_method(args, dag, ti)
@@ -298,7 +390,7 @@ def task_run(args, dag=None):
             _run_task_by_selected_method(args, dag, ti)
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 def task_failed_deps(args):
     """
     Returns the unmet dependencies for a task instance from the perspective of the
@@ -312,7 +404,7 @@ def task_failed_deps(args):
     """
     dag = get_dag(args.subdir, args.dag_id)
     task = dag.get_task(task_id=args.task_id)
-    ti = _get_ti(task, args.execution_date_or_run_id)
+    ti, _ = _get_ti(task, args.map_index, exec_date_or_run_id=args.execution_date_or_run_id)
 
     dep_context = DepContext(deps=SCHEDULER_QUEUED_DEPS)
     failed_deps = list(ti.get_failed_dep_statuses(dep_context=dep_context))
@@ -325,7 +417,7 @@ def task_failed_deps(args):
         print("Task instance dependencies are all met.")
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 @suppress_logs_and_warning
 def task_state(args):
     """
@@ -335,11 +427,11 @@ def task_state(args):
     """
     dag = get_dag(args.subdir, args.dag_id)
     task = dag.get_task(task_id=args.task_id)
-    ti = _get_ti(task, args.execution_date_or_run_id)
+    ti, _ = _get_ti(task, args.map_index, exec_date_or_run_id=args.execution_date_or_run_id)
     print(ti.current_state())
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 @suppress_logs_and_warning
 def task_list(args, dag=None):
     """Lists the tasks within a DAG at the command line"""
@@ -351,7 +443,7 @@ def task_list(args, dag=None):
         print("\n".join(tasks))
 
 
-SUPPORTED_DEBUGGER_MODULES: List[str] = [
+SUPPORTED_DEBUGGER_MODULES: list[str] = [
     "pudb",
     "web_pdb",
     "ipdb",
@@ -379,7 +471,7 @@ def _guess_debugger():
     return importlib.import_module("pdb")
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 @suppress_logs_and_warning
 @provide_session
 def task_states_for_dag_run(args, session=None):
@@ -406,21 +498,25 @@ def task_states_for_dag_run(args, session=None):
             "not found"
         )
 
-    AirflowConsole().print_as(
-        data=dag_run.task_instances,
-        output=args.output,
-        mapper=lambda ti: {
+    has_mapped_instances = any(ti.map_index >= 0 for ti in dag_run.task_instances)
+
+    def format_task_instance(ti: TaskInstance) -> dict[str, str]:
+        data = {
             "dag_id": ti.dag_id,
             "execution_date": dag_run.execution_date.isoformat(),
             "task_id": ti.task_id,
             "state": ti.state,
             "start_date": ti.start_date.isoformat() if ti.start_date else "",
             "end_date": ti.end_date.isoformat() if ti.end_date else "",
-        },
-    )
+        }
+        if has_mapped_instances:
+            data["map_index"] = str(ti.map_index) if ti.map_index >= 0 else ""
+        return data
+
+    AirflowConsole().print_as(data=dag_run.task_instances, output=args.output, mapper=format_task_instance)
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 def task_test(args, dag=None):
     """Tests task for a given dag_id"""
     # We want to log output from operators etc to show up here. Normally
@@ -454,13 +550,16 @@ def task_test(args, dag=None):
     if task.params:
         task.params.validate()
 
-    ti = _get_ti(task, args.execution_date_or_run_id, create_if_necssary=True)
+    ti, dr_created = _get_ti(
+        task, args.map_index, exec_date_or_run_id=args.execution_date_or_run_id, create_if_necessary="db"
+    )
 
     try:
-        if args.dry_run:
-            ti.dry_run()
-        else:
-            ti.run(ignore_task_deps=True, ignore_ti_state=True, test_mode=True)
+        with redirect_stdout(RedactedIO()):
+            if args.dry_run:
+                ti.dry_run()
+            else:
+                ti.run(ignore_task_deps=True, ignore_ti_state=True, test_mode=True)
     except Exception:
         if args.post_mortem:
             debugger = _guess_debugger()
@@ -472,15 +571,20 @@ def task_test(args, dag=None):
             # Make sure to reset back to normal. When run for CLI this doesn't
             # matter, but it does for test suite
             logging.getLogger('airflow.task').propagate = False
+        if dr_created:
+            with create_session() as session:
+                session.delete(ti.dag_run)
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 @suppress_logs_and_warning
 def task_render(args):
     """Renders and displays templated fields for a given task"""
     dag = get_dag(args.subdir, args.dag_id)
     task = dag.get_task(task_id=args.task_id)
-    ti = _get_ti(task, args.execution_date_or_run_id, create_if_necssary=True)
+    ti, _ = _get_ti(
+        task, args.map_index, exec_date_or_run_id=args.execution_date_or_run_id, create_if_necessary="memory"
+    )
     ti.render_templates()
     for attr in task.__class__.template_fields:
         print(
@@ -494,7 +598,7 @@ def task_render(args):
         )
 
 
-@cli_utils.action_logging
+@cli_utils.action_cli(check_db=False)
 def task_clear(args):
     """Clears all task instances or only those matched by regex for a DAG(s)"""
     logging.basicConfig(level=settings.LOGGING_LEVEL, format=settings.SIMPLE_LOG_FORMAT)
