@@ -21,6 +21,7 @@ KubernetesExecutor
     For more information on how the KubernetesExecutor works, take a look at the guide:
     :ref:`executor:KubernetesExecutor`
 """
+from __future__ import annotations
 
 import functools
 import json
@@ -28,14 +29,14 @@ import multiprocessing
 import time
 from datetime import timedelta
 from queue import Empty, Queue
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, Optional, Sequence, Tuple
 
 from kubernetes import client, watch
 from kubernetes.client import Configuration, models as k8s
 from kubernetes.client.rest import ApiException
 from urllib3.exceptions import ReadTimeoutError
 
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, PodReconciliationError
 from airflow.executors.base_executor import NOT_STARTED_MESSAGE, BaseExecutor, CommandType
 from airflow.kubernetes import pod_generator
 from airflow.kubernetes.kube_client import get_kube_client
@@ -77,10 +78,10 @@ class KubernetesJobWatcher(multiprocessing.Process, LoggingMixin):
 
     def __init__(
         self,
-        namespace: Optional[str],
+        namespace: str | None,
         multi_namespace_mode: bool,
-        watcher_queue: 'Queue[KubernetesWatchType]',
-        resource_version: Optional[str],
+        watcher_queue: Queue[KubernetesWatchType],
+        resource_version: str | None,
         scheduler_job_id: str,
         kube_config: Configuration,
     ):
@@ -121,10 +122,10 @@ class KubernetesJobWatcher(multiprocessing.Process, LoggingMixin):
     def _run(
         self,
         kube_client: client.CoreV1Api,
-        resource_version: Optional[str],
+        resource_version: str | None,
         scheduler_job_id: str,
         kube_config: Any,
-    ) -> Optional[str]:
+    ) -> str | None:
         self.log.info('Event: and now my watch begins starting at resource_version: %s', resource_version)
         watcher = watch.Watch()
 
@@ -135,7 +136,7 @@ class KubernetesJobWatcher(multiprocessing.Process, LoggingMixin):
             for key, value in kube_config.kube_client_request_args.items():
                 kwargs[key] = value
 
-        last_resource_version: Optional[str] = None
+        last_resource_version: str | None = None
         if self.multi_namespace_mode:
             list_worker_pods = functools.partial(
                 watcher.stream, kube_client.list_pod_for_all_namespaces, **kwargs
@@ -193,7 +194,7 @@ class KubernetesJobWatcher(multiprocessing.Process, LoggingMixin):
         pod_id: str,
         namespace: str,
         status: str,
-        annotations: Dict[str, str],
+        annotations: dict[str, str],
         resource_version: str,
         event: Any,
     ) -> None:
@@ -234,8 +235,8 @@ class AirflowKubernetesScheduler(LoggingMixin):
     def __init__(
         self,
         kube_config: Any,
-        task_queue: 'Queue[KubernetesJobType]',
-        result_queue: 'Queue[KubernetesResultsType]',
+        task_queue: Queue[KubernetesJobType],
+        result_queue: Queue[KubernetesResultsType],
         kube_client: client.CoreV1Api,
         scheduler_job_id: str,
     ):
@@ -300,8 +301,9 @@ class AirflowKubernetesScheduler(LoggingMixin):
         and store relevant info in the current_jobs map so we can track the job's
         status
         """
-        self.log.info('Kubernetes job is %s', str(next_job).replace("\n", " "))
         key, command, kube_executor_config, pod_template_file = next_job
+        self.log.info('Kubernetes job is %s', key)
+
         dag_id, task_id, run_id, try_number, map_index = key
 
         if command[0:3] != ["airflow", "tasks", "run"]:
@@ -411,7 +413,7 @@ class AirflowKubernetesScheduler(LoggingMixin):
         self._manager.shutdown()
 
 
-def get_base_pod_from_template(pod_template_file: Optional[str], kube_config: Any) -> k8s.V1Pod:
+def get_base_pod_from_template(pod_template_file: str | None, kube_config: Any) -> k8s.V1Pod:
     """
     Reads either the pod_template_file set in the executor_config or the base pod_template_file
     set in the airflow.cfg to craft a "base pod" that will be used by the KubernetesExecutor
@@ -434,14 +436,14 @@ class KubernetesExecutor(BaseExecutor):
     def __init__(self):
         self.kube_config = KubeConfig()
         self._manager = multiprocessing.Manager()
-        self.task_queue: 'Queue[KubernetesJobType]' = self._manager.Queue()
-        self.result_queue: 'Queue[KubernetesResultsType]' = self._manager.Queue()
-        self.kube_scheduler: Optional[AirflowKubernetesScheduler] = None
-        self.kube_client: Optional[client.CoreV1Api] = None
-        self.scheduler_job_id: Optional[str] = None
-        self.event_scheduler: Optional[EventScheduler] = None
-        self.last_handled: Dict[TaskInstanceKey, float] = {}
-        self.kubernetes_queue: Optional[str] = None
+        self.task_queue: Queue[KubernetesJobType] = self._manager.Queue()
+        self.result_queue: Queue[KubernetesResultsType] = self._manager.Queue()
+        self.kube_scheduler: AirflowKubernetesScheduler | None = None
+        self.kube_client: client.CoreV1Api | None = None
+        self.scheduler_job_id: str | None = None
+        self.event_scheduler: EventScheduler | None = None
+        self.last_handled: dict[TaskInstanceKey, float] = {}
+        self.kubernetes_queue: str | None = None
         super().__init__(parallelism=self.kube_config.parallelism)
 
     @provide_session
@@ -464,7 +466,7 @@ class KubernetesExecutor(BaseExecutor):
         query = session.query(TaskInstance).filter(TaskInstance.state == State.QUEUED)
         if self.kubernetes_queue:
             query = query.filter(TaskInstance.queue == self.kubernetes_queue)
-        queued_tis: List[TaskInstance] = query.all()
+        queued_tis: list[TaskInstance] = query.all()
         self.log.info('Found %s queued task instances', len(queued_tis))
 
         # Go through the "last seen" dictionary and clean out old entries
@@ -542,8 +544,8 @@ class KubernetesExecutor(BaseExecutor):
         self,
         key: TaskInstanceKey,
         command: CommandType,
-        queue: Optional[str] = None,
-        executor_config: Optional[Any] = None,
+        queue: str | None = None,
+        executor_config: Any | None = None,
     ) -> None:
         """Executes task asynchronously"""
         self.log.info('Add task %s with command %s with executor_config %s', key, command, executor_config)
@@ -617,6 +619,13 @@ class KubernetesExecutor(BaseExecutor):
                 task = self.task_queue.get_nowait()
                 try:
                     self.kube_scheduler.run_next(task)
+                except PodReconciliationError as e:
+                    self.log.error(
+                        "Pod reconciliation failed, likely due to kubernetes library upgrade. "
+                        "Try clearing the task to re-run.",
+                        exc_info=True,
+                    )
+                    self.fail(task[0], e)
                 except ApiException as e:
 
                     # These codes indicate something is wrong with pod definition; otherwise we assume pod
@@ -677,7 +686,7 @@ class KubernetesExecutor(BaseExecutor):
                 )
                 self.kube_scheduler.delete_pod(pod.metadata.name, pod.metadata.namespace)
 
-    def _change_state(self, key: TaskInstanceKey, state: Optional[str], pod_id: str, namespace: str) -> None:
+    def _change_state(self, key: TaskInstanceKey, state: str | None, pod_id: str, namespace: str) -> None:
         if state != State.RUNNING:
             if self.kube_config.delete_worker_pods:
                 if not self.kube_scheduler:
@@ -691,7 +700,7 @@ class KubernetesExecutor(BaseExecutor):
                 self.log.debug('Could not find key: %s', str(key))
         self.event_buffer[key] = state, None
 
-    def try_adopt_task_instances(self, tis: List[TaskInstance]) -> List[TaskInstance]:
+    def try_adopt_task_instances(self, tis: Sequence[TaskInstance]) -> Sequence[TaskInstance]:
         tis_to_flush = [ti for ti in tis if not ti.queued_by_job_id]
         scheduler_job_ids = {ti.queued_by_job_id for ti in tis}
         pod_ids = {ti.key: ti for ti in tis if ti.queued_by_job_id}
@@ -707,7 +716,7 @@ class KubernetesExecutor(BaseExecutor):
         return tis_to_flush
 
     def adopt_launched_task(
-        self, kube_client: client.CoreV1Api, pod: k8s.V1Pod, pod_ids: Dict[TaskInstanceKey, k8s.V1Pod]
+        self, kube_client: client.CoreV1Api, pod: k8s.V1Pod, pod_ids: dict[TaskInstanceKey, k8s.V1Pod]
     ) -> None:
         """
         Patch existing pod so that the current KubernetesJobWatcher can monitor it via label selectors
