@@ -15,10 +15,9 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
+from __future__ import annotations
 
 from time import sleep
-from typing import Optional
 
 from sqlalchemy import Column, Index, Integer, String
 from sqlalchemy.exc import OperationalError
@@ -29,9 +28,7 @@ from airflow.compat.functools import cached_property
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException
 from airflow.executors.executor_loader import ExecutorLoader
-from airflow.models import DagRun
 from airflow.models.base import ID_LEN, Base
-from airflow.models.taskinstance import TaskInstance
 from airflow.stats import Stats
 from airflow.utils import timezone
 from airflow.utils.helpers import convert_camel_to_snake
@@ -43,6 +40,12 @@ from airflow.utils.sqlalchemy import UtcDateTime
 from airflow.utils.state import State
 
 
+def _resolve_dagrun_model():
+    from airflow.models.dagrun import DagRun
+
+    return DagRun
+
+
 class BaseJob(Base, LoggingMixin):
     """
     Abstract class to be derived for jobs. Jobs are processing items with state
@@ -75,15 +78,15 @@ class BaseJob(Base, LoggingMixin):
     )
 
     task_instances_enqueued = relationship(
-        TaskInstance,
-        primaryjoin=id == foreign(TaskInstance.queued_by_job_id),
+        "TaskInstance",
+        primaryjoin="BaseJob.id == foreign(TaskInstance.queued_by_job_id)",
         backref=backref('queued_by_job', uselist=False),
     )
 
     dag_runs = relationship(
-        DagRun,
-        primaryjoin=id == foreign(DagRun.creating_job_id),
-        backref=backref('creating_job'),
+        "DagRun",
+        primaryjoin=lambda: BaseJob.id == foreign(_resolve_dagrun_model().creating_job_id),
+        backref='creating_job',
     )
 
     """
@@ -106,7 +109,7 @@ class BaseJob(Base, LoggingMixin):
         if heartrate is not None:
             self.heartrate = heartrate
         self.unixname = getuser()
-        self.max_tis_per_query = conf.getint('scheduler', 'max_tis_per_query')
+        self.max_tis_per_query: int = conf.getint('scheduler', 'max_tis_per_query')
         super().__init__(*args, **kwargs)
 
     @cached_property
@@ -115,7 +118,7 @@ class BaseJob(Base, LoggingMixin):
 
     @classmethod
     @provide_session
-    def most_recent_job(cls, session=None) -> Optional['BaseJob']:
+    def most_recent_job(cls, session=None) -> BaseJob | None:
         """
         Return the most recent job of this type, if any, based on last
         heartbeat received.
@@ -137,7 +140,6 @@ class BaseJob(Base, LoggingMixin):
 
         :param grace_multiplier: multiplier of heartrate to require heart beat
             within
-        :type grace_multiplier: number
         :rtype: boolean
         """
         return (
@@ -186,7 +188,6 @@ class BaseJob(Base, LoggingMixin):
 
         :param only_if_necessary: If the heartbeat is not yet due then do
             nothing (don't update column, don't call ``heartbeat_callback``)
-        :type only_if_necessary: boolean
         """
         seconds_remaining = 0
         if self.latest_heartbeat:
