@@ -15,7 +15,6 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Change ``TaskInstance`` and ``TaskReschedule`` tables from execution_date to run_id.
 
 Revision ID: 7b2661a43ba3
@@ -23,6 +22,7 @@ Revises: 142555e44c17
 Create Date: 2021-07-15 15:26:12.710749
 
 """
+from __future__ import annotations
 
 import sqlalchemy as sa
 from alembic import op
@@ -92,8 +92,10 @@ def upgrade():
                 'run_id', existing_type=sa.String(length=ID_LEN), type_=string_id_col_type, nullable=False
             )
             batch_op.alter_column('execution_date', existing_type=dt_type, nullable=False)
-            batch_op.drop_constraint('dag_id', 'unique')
-            batch_op.drop_constraint('dag_id_2', 'unique')
+            inspector = sa.inspect(conn.engine)
+            unique_keys = inspector.get_unique_constraints('dag_run')
+            for unique_key in unique_keys:
+                batch_op.drop_constraint(unique_key['name'], type_='unique')
             batch_op.create_unique_constraint(
                 'dag_run_dag_id_execution_date_key', ['dag_id', 'execution_date']
             )
@@ -341,6 +343,10 @@ def downgrade():
             ['dag_id', 'task_id', 'execution_date'],
             ondelete='CASCADE',
         )
+        if dialect_name == "mysql":
+            batch_op.create_index(
+                'task_reschedule_dag_task_date_fkey', ['dag_id', 'execution_date'], unique=False
+            )
 
     if dialect_name == "mssql":
 
