@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
@@ -16,42 +15,50 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
-from builtins import ImportError as BuiltinImportError
+from __future__ import annotations
 
 import logging
+from typing import Optional
 
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException
+from airflow.typing_compat import Protocol
 
 log = logging.getLogger(__name__)
 
 
-class InvalidFernetToken(Exception):
-    # If Fernet isn't loaded we need a valid exception class to catch. If it is
-    # loaded this will get reset to the actual class once get_fernet() is called
-    pass
+class FernetProtocol(Protocol):
+    """This class is only used for TypeChecking (for IDEs, mypy, etc)"""
+
+    def decrypt(self, b):
+        """Decrypt with Fernet"""
+
+    def encrypt(self, b):
+        """Encrypt with Fernet"""
 
 
-class NullFernet(object):
+class NullFernet:
     """
     A "Null" encryptor class that doesn't encrypt or decrypt but that presents
     a similar interface to Fernet.
 
     The purpose of this is to make the rest of the code not have to know the
     difference, and to only display the message once, not 20 times when
-    `airflow initdb` is ran.
+    `airflow db init` is ran.
     """
+
     is_encrypted = False
 
     def decrypt(self, b):
+        """Decrypt with Fernet."""
         return b
 
     def encrypt(self, b):
+        """Encrypt with Fernet."""
         return b
 
 
-_fernet = None
+_fernet = None  # type: Optional[FernetProtocol]
 
 
 def get_fernet():
@@ -64,36 +71,24 @@ def get_fernet():
     :return: Fernet object
     :raises: airflow.exceptions.AirflowException if there's a problem trying to load Fernet
     """
+    from cryptography.fernet import Fernet, MultiFernet
+
     global _fernet
 
     if _fernet:
         return _fernet
-    try:
-        from cryptography.fernet import Fernet, MultiFernet, InvalidToken
-        global InvalidFernetToken
-        InvalidFernetToken = InvalidToken
-
-    except BuiltinImportError:
-        log.warning(
-            "cryptography not found - values will not be stored encrypted."
-        )
-        _fernet = NullFernet()
-        return _fernet
 
     try:
         fernet_key = conf.get('core', 'FERNET_KEY')
         if not fernet_key:
-            log.warning(
-                "empty cryptography key - values will not be stored encrypted."
-            )
+            log.warning("empty cryptography key - values will not be stored encrypted.")
             _fernet = NullFernet()
         else:
-            _fernet = MultiFernet([
-                Fernet(fernet_part.encode('utf-8'))
-                for fernet_part in fernet_key.split(',')
-            ])
+            _fernet = MultiFernet(
+                [Fernet(fernet_part.encode('utf-8')) for fernet_part in fernet_key.split(',')]
+            )
             _fernet.is_encrypted = True
-    except (ValueError, TypeError) as ve:
-        raise AirflowException("Could not create Fernet object: {}".format(ve))
+    except (ValueError, TypeError) as value_error:
+        raise AirflowException(f"Could not create Fernet object: {value_error}")
 
     return _fernet
