@@ -14,10 +14,13 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import warnings
 from abc import ABC
-from typing import TYPE_CHECKING, List, Optional
+from typing import TYPE_CHECKING
+
+from airflow.exceptions import RemovedInAirflow3Warning
 
 if TYPE_CHECKING:
     from airflow.models.connection import Connection
@@ -26,58 +29,103 @@ if TYPE_CHECKING:
 class BaseSecretsBackend(ABC):
     """Abstract base class to retrieve Connection object given a conn_id or Variable given a key"""
 
-    def __init__(self, **kwargs):
-        pass
-
     @staticmethod
     def build_path(path_prefix: str, secret_id: str, sep: str = "/") -> str:
         """
         Given conn_id, build path for Secrets Backend
 
         :param path_prefix: Prefix of the path to get secret
-        :type path_prefix: str
         :param secret_id: Secret id
-        :type secret_id: str
         :param sep: separator used to concatenate connections_prefix and conn_id. Default: "/"
-        :type sep: str
         """
         return f"{path_prefix}{sep}{secret_id}"
 
-    def get_conn_uri(self, conn_id: str) -> Optional[str]:
+    def get_conn_value(self, conn_id: str) -> str | None:
         """
-        Get conn_uri from Secrets Backend
+        Retrieve from Secrets Backend a string value representing the Connection object.
+
+        If the client your secrets backend uses already returns a python dict, you should override
+        ``get_connection`` instead.
 
         :param conn_id: connection id
-        :type conn_id: str
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
-    def get_connection(self, conn_id: str) -> Optional['Connection']:
+    def deserialize_connection(self, conn_id: str, value: str) -> Connection:
         """
-        Return connection object with a given ``conn_id``.
+        Given a serialized representation of the airflow Connection, return an instance.
+        Looks at first character to determine how to deserialize.
 
         :param conn_id: connection id
-        :type conn_id: str
+        :param value: the serialized representation of the Connection object
+        :return: the deserialized Connection
         """
         from airflow.models.connection import Connection
 
-        conn_uri = self.get_conn_uri(conn_id=conn_id)
-        if not conn_uri:
-            return None
-        conn = Connection(conn_id=conn_id, uri=conn_uri)
-        return conn
+        value = value.strip()
+        if value[0] == '{':
+            return Connection.from_json(conn_id=conn_id, value=value)
+        else:
+            return Connection(conn_id=conn_id, uri=value)
 
-    def get_connections(self, conn_id: str) -> List['Connection']:
+    def get_conn_uri(self, conn_id: str) -> str | None:
+        """
+        Get conn_uri from Secrets Backend
+
+        This method is deprecated and will be removed in a future release; implement ``get_conn_value``
+        instead.
+
+        :param conn_id: connection id
+        """
+        raise NotImplementedError()
+
+    def get_connection(self, conn_id: str) -> Connection | None:
+        """
+        Return connection object with a given ``conn_id``.
+
+        Tries ``get_conn_value`` first and if not implemented, tries ``get_conn_uri``
+
+        :param conn_id: connection id
+        """
+        value = None
+
+        not_implemented_get_conn_value = False
+        # TODO: after removal of ``get_conn_uri`` we should not catch NotImplementedError here
+        try:
+            value = self.get_conn_value(conn_id=conn_id)
+        except NotImplementedError:
+            not_implemented_get_conn_value = True
+            warnings.warn(
+                "Method `get_conn_uri` is deprecated. Please use `get_conn_value`.",
+                RemovedInAirflow3Warning,
+                stacklevel=2,
+            )
+
+        if not_implemented_get_conn_value:
+            try:
+                value = self.get_conn_uri(conn_id=conn_id)
+            except NotImplementedError:
+                raise NotImplementedError(
+                    f"Secrets backend {self.__class__.__name__} neither implements "
+                    "`get_conn_value` nor `get_conn_uri`.  Method `get_conn_uri` is "
+                    "deprecated and will be removed in a future release. Please implement `get_conn_value`."
+                )
+
+        if value:
+            return self.deserialize_connection(conn_id=conn_id, value=value)
+        else:
+            return None
+
+    def get_connections(self, conn_id: str) -> list[Connection]:
         """
         Return connection object with a given ``conn_id``.
 
         :param conn_id: connection id
-        :type conn_id: str
         """
         warnings.warn(
             "This method is deprecated. Please use "
             "`airflow.secrets.base_secrets.BaseSecretsBackend.get_connection`.",
-            PendingDeprecationWarning,
+            RemovedInAirflow3Warning,
             stacklevel=2,
         )
         conn = self.get_connection(conn_id=conn_id)
@@ -85,17 +133,16 @@ class BaseSecretsBackend(ABC):
             return [conn]
         return []
 
-    def get_variable(self, key: str) -> Optional[str]:
+    def get_variable(self, key: str) -> str | None:
         """
         Return value for Airflow Variable
 
         :param key: Variable Key
-        :type key: str
         :return: Variable Value
         """
         raise NotImplementedError()
 
-    def get_config(self, key: str) -> Optional[str]:
+    def get_config(self, key: str) -> str | None:
         """
         Return value for Airflow Config Key
 
