@@ -15,6 +15,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 from collections import Counter
 from typing import TYPE_CHECKING
@@ -58,6 +59,7 @@ class TriggerRuleDep(BaseTIDep):
             counter.get(State.SKIPPED, 0),
             counter.get(State.FAILED, 0),
             counter.get(State.UPSTREAM_FAILED, 0),
+            counter.get(State.REMOVED, 0),
             sum(counter.values()),
         )
 
@@ -72,7 +74,7 @@ class TriggerRuleDep(BaseTIDep):
             yield self._passing_status(reason="The task had a always trigger rule set.")
             return
         # see if the task name is in the task upstream for our task
-        successes, skipped, failed, upstream_failed, done = self._get_states_count_upstream_ti(
+        successes, skipped, failed, upstream_failed, removed, done = self._get_states_count_upstream_ti(
             task=ti.task, finished_tis=dep_context.ensure_finished_tis(ti.get_dagrun(session), session)
         )
 
@@ -82,13 +84,15 @@ class TriggerRuleDep(BaseTIDep):
             skipped=skipped,
             failed=failed,
             upstream_failed=upstream_failed,
+            removed=removed,
             done=done,
             flag_upstream_failed=dep_context.flag_upstream_failed,
+            dep_context=dep_context,
             session=session,
         )
 
     @staticmethod
-    def _count_upstreams(ti: "TaskInstance", *, session: "Session"):
+    def _count_upstreams(ti: TaskInstance, *, session: Session):
         from airflow.models.taskinstance import TaskInstance
 
         # Optimization: Don't need to hit the database if no upstreams are mapped.
@@ -115,14 +119,16 @@ class TriggerRuleDep(BaseTIDep):
     @provide_session
     def _evaluate_trigger_rule(
         self,
-        ti,
+        ti: TaskInstance,
         successes,
         skipped,
         failed,
         upstream_failed,
+        removed,
         done,
         flag_upstream_failed,
-        session: "Session" = NEW_SESSION,
+        dep_context: DepContext,
+        session: Session = NEW_SESSION,
     ):
         """
         Yields a dependency status that indicate whether the given task instance's trigger
@@ -142,53 +148,57 @@ class TriggerRuleDep(BaseTIDep):
         """
         task = ti.task
         upstream = self._count_upstreams(ti, session=session)
-        trigger_rule: TR = task.trigger_rule
+        trigger_rule = task.trigger_rule
         upstream_done = done >= upstream
         upstream_tasks_state = {
             "total": upstream,
             "successes": successes,
             "skipped": skipped,
             "failed": failed,
+            "removed": removed,
             "upstream_failed": upstream_failed,
             "done": done,
         }
-        # TODO(aoen): Ideally each individual trigger rules would be its own class, but
-        # this isn't very feasible at the moment since the database queries need to be
-        # bundled together for efficiency.
-        # handling instant state assignment based on trigger rules
+        changed: bool = False
         if flag_upstream_failed:
             if trigger_rule == TR.ALL_SUCCESS:
                 if upstream_failed or failed:
-                    ti.set_state(State.UPSTREAM_FAILED, session)
+                    changed = ti.set_state(State.UPSTREAM_FAILED, session)
                 elif skipped:
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
+                elif removed and successes and ti.map_index > -1:
+                    if ti.map_index >= successes:
+                        changed = ti.set_state(State.REMOVED, session)
             elif trigger_rule == TR.ALL_FAILED:
                 if successes or skipped:
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
             elif trigger_rule == TR.ONE_SUCCESS:
                 if upstream_done and done == skipped:
                     # if upstream is done and all are skipped mark as skipped
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
                 elif upstream_done and successes <= 0:
                     # if upstream is done and there are no successes mark as upstream failed
-                    ti.set_state(State.UPSTREAM_FAILED, session)
+                    changed = ti.set_state(State.UPSTREAM_FAILED, session)
             elif trigger_rule == TR.ONE_FAILED:
                 if upstream_done and not (failed or upstream_failed):
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
             elif trigger_rule == TR.NONE_FAILED:
                 if upstream_failed or failed:
-                    ti.set_state(State.UPSTREAM_FAILED, session)
+                    changed = ti.set_state(State.UPSTREAM_FAILED, session)
             elif trigger_rule == TR.NONE_FAILED_MIN_ONE_SUCCESS:
                 if upstream_failed or failed:
-                    ti.set_state(State.UPSTREAM_FAILED, session)
+                    changed = ti.set_state(State.UPSTREAM_FAILED, session)
                 elif skipped == upstream:
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
             elif trigger_rule == TR.NONE_SKIPPED:
                 if skipped:
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
             elif trigger_rule == TR.ALL_SKIPPED:
                 if successes or failed:
-                    ti.set_state(State.SKIPPED, session)
+                    changed = ti.set_state(State.SKIPPED, session)
+
+        if changed:
+            dep_context.have_changed_ti_states = True
 
         if trigger_rule == TR.ONE_SUCCESS:
             if successes <= 0:
@@ -210,6 +220,8 @@ class TriggerRuleDep(BaseTIDep):
                 )
         elif trigger_rule == TR.ALL_SUCCESS:
             num_failures = upstream - successes
+            if ti.map_index > -1:
+                num_failures -= removed
             if num_failures > 0:
                 yield self._failing_status(
                     reason=(
@@ -221,6 +233,8 @@ class TriggerRuleDep(BaseTIDep):
                 )
         elif trigger_rule == TR.ALL_FAILED:
             num_successes = upstream - failed - upstream_failed
+            if ti.map_index > -1:
+                num_successes -= removed
             if num_successes > 0:
                 yield self._failing_status(
                     reason=(
@@ -242,6 +256,8 @@ class TriggerRuleDep(BaseTIDep):
                 )
         elif trigger_rule == TR.NONE_FAILED:
             num_failures = upstream - successes - skipped
+            if ti.map_index > -1:
+                num_failures -= removed
             if num_failures > 0:
                 yield self._failing_status(
                     reason=(
@@ -253,6 +269,8 @@ class TriggerRuleDep(BaseTIDep):
                 )
         elif trigger_rule == TR.NONE_FAILED_MIN_ONE_SUCCESS:
             num_failures = upstream - successes - skipped
+            if ti.map_index > -1:
+                num_failures -= removed
             if num_failures > 0:
                 yield self._failing_status(
                     reason=(
