@@ -15,6 +15,8 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import io
 import logging
 import os
@@ -22,12 +24,13 @@ import re
 import zipfile
 from collections import OrderedDict
 from pathlib import Path
-from typing import TYPE_CHECKING, Dict, Generator, List, NamedTuple, Optional, Pattern, Type, Union, overload
+from typing import TYPE_CHECKING, Generator, NamedTuple, Pattern, overload
 
 from pathspec.patterns import GitWildMatchPattern
 from typing_extensions import Protocol
 
 from airflow.configuration import conf
+from airflow.exceptions import RemovedInAirflow3Warning
 
 if TYPE_CHECKING:
     import pathlib
@@ -39,14 +42,14 @@ class _IgnoreRule(Protocol):
     """Interface for ignore rules for structural subtyping"""
 
     @staticmethod
-    def compile(pattern: str, base_dir: Path, definition_file: Path) -> Optional['_IgnoreRule']:
+    def compile(pattern: str, base_dir: Path, definition_file: Path) -> _IgnoreRule | None:
         """
         Build an ignore rule from the supplied pattern where base_dir
         and definition_file should be absolute paths.
         """
 
     @staticmethod
-    def match(path: Path, rules: List['_IgnoreRule']) -> bool:
+    def match(path: Path, rules: list[_IgnoreRule]) -> bool:
         """Match a candidate absolute path against a list of rules"""
 
 
@@ -57,7 +60,7 @@ class _RegexpIgnoreRule(NamedTuple):
     base_dir: Path
 
     @staticmethod
-    def compile(pattern: str, base_dir: Path, definition_file: Path) -> Optional[_IgnoreRule]:
+    def compile(pattern: str, base_dir: Path, definition_file: Path) -> _IgnoreRule | None:
         """Build an ignore rule from the supplied regexp pattern and log a useful warning if it is invalid"""
         try:
             return _RegexpIgnoreRule(re.compile(pattern), base_dir)
@@ -66,7 +69,7 @@ class _RegexpIgnoreRule(NamedTuple):
             return None
 
     @staticmethod
-    def match(path: Path, rules: List[_IgnoreRule]) -> bool:
+    def match(path: Path, rules: list[_IgnoreRule]) -> bool:
         """Match a list of ignore rules against the supplied path"""
         for rule in rules:
             if not isinstance(rule, _RegexpIgnoreRule):
@@ -81,13 +84,13 @@ class _GlobIgnoreRule(NamedTuple):
 
     pattern: Pattern
     raw_pattern: str
-    include: Optional[bool] = None
-    relative_to: Optional[Path] = None
+    include: bool | None = None
+    relative_to: Path | None = None
 
     @staticmethod
-    def compile(pattern: str, _, definition_file: Path) -> Optional[_IgnoreRule]:
+    def compile(pattern: str, _, definition_file: Path) -> _IgnoreRule | None:
         """Build an ignore rule from the supplied glob pattern and log a useful warning if it is invalid"""
-        relative_to: Optional[Path] = None
+        relative_to: Path | None = None
         if pattern.strip() == "/":
             # "/" doesn't match anything in gitignore
             log.warning("Ignoring no-op glob pattern '/' from %s", definition_file)
@@ -102,7 +105,7 @@ class _GlobIgnoreRule(NamedTuple):
         return _GlobIgnoreRule(ignore_pattern.regex, pattern, ignore_pattern.include, relative_to)
 
     @staticmethod
-    def match(path: Path, rules: List[_IgnoreRule]) -> bool:
+    def match(path: Path, rules: list[_IgnoreRule]) -> bool:
         """Match a list of ignore rules against the supplied path"""
         matched = False
         for r in rules:
@@ -125,7 +128,7 @@ def TemporaryDirectory(*args, **kwargs):
 
     warnings.warn(
         "This function is deprecated. Please use `tempfile.TemporaryDirectory`",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
         stacklevel=2,
     )
 
@@ -144,7 +147,7 @@ def mkdirs(path, mode):
 
     warnings.warn(
         f"This function is deprecated. Please use `pathlib.Path({path}).mkdir`",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
         stacklevel=2,
     )
     Path(path).mkdir(mode=mode, parents=True, exist_ok=True)
@@ -159,11 +162,11 @@ def correct_maybe_zipped(fileloc: None) -> None:
 
 
 @overload
-def correct_maybe_zipped(fileloc: Union[str, Path]) -> Union[str, Path]:
+def correct_maybe_zipped(fileloc: str | Path) -> str | Path:
     ...
 
 
-def correct_maybe_zipped(fileloc: Union[None, str, Path]) -> Union[None, str, Path]:
+def correct_maybe_zipped(fileloc: None | str | Path) -> None | str | Path:
     """
     If the path contains a folder with a .zip suffix, then
     the folder is treated as a zip archive and path to zip is returned.
@@ -198,7 +201,7 @@ def open_maybe_zipped(fileloc, mode='r'):
 def _find_path_from_directory(
     base_dir_path: str,
     ignore_file_name: str,
-    ignore_rule_type: Type[_IgnoreRule],
+    ignore_rule_type: type[_IgnoreRule],
 ) -> Generator[str, None, None]:
     """
     Recursively search the base path and return the list of file paths that should not be ignored by
@@ -210,10 +213,10 @@ def _find_path_from_directory(
     :return: a generator of file paths which should not be ignored.
     """
     # A Dict of patterns, keyed using resolved, absolute paths
-    patterns_by_dir: Dict[Path, List[_IgnoreRule]] = {}
+    patterns_by_dir: dict[Path, list[_IgnoreRule]] = {}
 
     for root, dirs, files in os.walk(base_dir_path, followlinks=True):
-        patterns: List[_IgnoreRule] = patterns_by_dir.get(Path(root).resolve(), [])
+        patterns: list[_IgnoreRule] = patterns_by_dir.get(Path(root).resolve(), [])
 
         ignore_file_path = Path(root) / ignore_file_name
         if ignore_file_path.is_file():
@@ -241,7 +244,7 @@ def _find_path_from_directory(
             if dirpath in patterns_by_dir:
                 raise RuntimeError(
                     "Detected recursive loop when walking DAG directory "
-                    + f"{base_dir_path}: {dirpath} has appeared more than once."
+                    f"{base_dir_path}: {dirpath} has appeared more than once."
                 )
             patterns_by_dir.update({dirpath: patterns.copy()})
 
@@ -276,10 +279,9 @@ def find_path_from_directory(
 
 
 def list_py_file_paths(
-    directory: Union[str, "pathlib.Path"],
+    directory: str | pathlib.Path,
     safe_mode: bool = conf.getboolean('core', 'DAG_DISCOVERY_SAFE_MODE', fallback=True),
-    include_examples: Optional[bool] = None,
-    include_smart_sensor: Optional[bool] = conf.getboolean('smart_sensor', 'use_smart_sensor'),
+    include_examples: bool | None = None,
 ):
     """
     Traverse a directory and look for Python files.
@@ -290,13 +292,12 @@ def list_py_file_paths(
         core.DAG_DISCOVERY_SAFE_MODE configuration setting. If not set, default
         to safe.
     :param include_examples: include example DAGs
-    :param include_smart_sensor: include smart sensor native control DAGs
     :return: a list of paths to Python files in the specified directory
     :rtype: list[unicode]
     """
     if include_examples is None:
         include_examples = conf.getboolean('core', 'LOAD_EXAMPLES')
-    file_paths: List[str] = []
+    file_paths: list[str] = []
     if directory is None:
         file_paths = []
     elif os.path.isfile(directory):
@@ -307,16 +308,11 @@ def list_py_file_paths(
         from airflow import example_dags
 
         example_dag_folder = example_dags.__path__[0]  # type: ignore
-        file_paths.extend(list_py_file_paths(example_dag_folder, safe_mode, False, False))
-    if include_smart_sensor:
-        from airflow import smart_sensor_dags
-
-        smart_sensor_dag_folder = smart_sensor_dags.__path__[0]  # type: ignore
-        file_paths.extend(list_py_file_paths(smart_sensor_dag_folder, safe_mode, False, False))
+        file_paths.extend(list_py_file_paths(example_dag_folder, safe_mode, include_examples=False))
     return file_paths
 
 
-def find_dag_file_paths(directory: Union[str, "pathlib.Path"], safe_mode: bool) -> List[str]:
+def find_dag_file_paths(directory: str | pathlib.Path, safe_mode: bool) -> list[str]:
     """Finds file paths of all DAG files."""
     file_paths = []
 
@@ -340,7 +336,7 @@ def find_dag_file_paths(directory: Union[str, "pathlib.Path"], safe_mode: bool)
 COMMENT_PATTERN = re.compile(r"\s*#.*")
 
 
-def might_contain_dag(file_path: str, safe_mode: bool, zip_file: Optional[zipfile.ZipFile] = None):
+def might_contain_dag(file_path: str, safe_mode: bool, zip_file: zipfile.ZipFile | None = None):
     """
     Heuristic that guesses whether a Python file contains an Airflow DAG definition.
 
