@@ -15,21 +15,23 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
+import copy
 import datetime
 import json
 import logging
-from typing import Any, Dict, Iterable, Tuple
+from typing import Any, Iterable
 
 import pendulum
 from dateutil import relativedelta
-from sqlalchemy import PickleType, and_, event, false, nullsfirst, or_, tuple_
-from sqlalchemy.dialects import mssql
+from sqlalchemy import TIMESTAMP, PickleType, and_, event, false, nullsfirst, or_, tuple_
+from sqlalchemy.dialects import mssql, mysql
 from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.session import Session
 from sqlalchemy.sql import ColumnElement
 from sqlalchemy.sql.expression import ColumnOperators
-from sqlalchemy.types import JSON, DateTime, Text, TypeDecorator, TypeEngine, UnicodeText
+from sqlalchemy.types import JSON, Text, TypeDecorator, TypeEngine, UnicodeText
 
 from airflow import settings
 from airflow.configuration import conf
@@ -44,21 +46,21 @@ using_mysql = conf.get_mandatory_value('database', 'sql_alchemy_conn').lower().s
 
 class UtcDateTime(TypeDecorator):
     """
-    Almost equivalent to :class:`~sqlalchemy.types.DateTime` with
+    Almost equivalent to :class:`~sqlalchemy.types.TIMESTAMP` with
     ``timezone=True`` option, but it differs from that by:
 
     - Never silently take naive :class:`~datetime.datetime`, instead it
       always raise :exc:`ValueError` unless time zone aware value.
     - :class:`~datetime.datetime` value's :attr:`~datetime.datetime.tzinfo`
       is always converted to UTC.
-    - Unlike SQLAlchemy's built-in :class:`~sqlalchemy.types.DateTime`,
+    - Unlike SQLAlchemy's built-in :class:`~sqlalchemy.types.TIMESTAMP`,
       it never return naive :class:`~datetime.datetime`, but time zone
       aware value, even with SQLite or MySQL.
-    - Always returns DateTime in UTC
+    - Always returns TIMESTAMP in UTC
 
     """
 
-    impl = DateTime(timezone=True)
+    impl = TIMESTAMP(timezone=True)
 
     cache_ok = True
 
@@ -99,6 +101,8 @@ class UtcDateTime(TypeDecorator):
     def load_dialect_impl(self, dialect):
         if dialect.name == 'mssql':
             return mssql.DATETIME2(precision=6)
+        elif dialect.name == 'mysql':
+            return mysql.TIMESTAMP(fsp=6)
         return super().load_dialect_impl(dialect)
 
 
@@ -116,7 +120,7 @@ class ExtendedJSON(TypeDecorator):
         """Checks if the database supports JSON (i.e. is NOT MSSQL)"""
         return not conf.get("database", "sql_alchemy_conn").startswith("mssql")
 
-    def load_dialect_impl(self, dialect) -> "TypeEngine":
+    def load_dialect_impl(self, dialect) -> TypeEngine:
         if self.db_supports_json():
             return dialect.type_descriptor(JSON)
         return dialect.type_descriptor(UnicodeText)
@@ -128,7 +132,7 @@ class ExtendedJSON(TypeDecorator):
             return None
 
         # First, encode it into our custom JSON-targeted dict format
-        value = BaseSerialization._serialize(value)
+        value = BaseSerialization.serialize(value)
 
         # Then, if the database does not have native JSON support, encode it again as a string
         if not self.db_supports_json():
@@ -146,7 +150,7 @@ class ExtendedJSON(TypeDecorator):
         if not self.db_supports_json():
             value = json.loads(value)
 
-        return BaseSerialization._deserialize(value)
+        return BaseSerialization.deserialize(value)
 
 
 class ExecutorConfigType(PickleType):
@@ -157,6 +161,8 @@ class ExecutorConfigType(PickleType):
     Airflow's serializer before pickling.
     """
 
+    cache_ok = True
+
     def bind_processor(self, dialect):
 
         from airflow.serialization.serialized_objects import BaseSerialization
@@ -164,9 +170,10 @@ class ExecutorConfigType(PickleType):
         super_process = super().bind_processor(dialect)
 
         def process(value):
-            if isinstance(value, dict) and 'pod_override' in value:
-                value['pod_override'] = BaseSerialization()._serialize(value['pod_override'])
-            return super_process(value)
+            val_copy = copy.copy(value)
+            if isinstance(val_copy, dict) and 'pod_override' in val_copy:
+                val_copy['pod_override'] = BaseSerialization.serialize(val_copy['pod_override'])
+            return super_process(val_copy)
 
         return process
 
@@ -183,7 +190,7 @@ class ExecutorConfigType(PickleType):
 
                 # If pod_override was serialized with Airflow's BaseSerialization, deserialize it
                 if isinstance(pod_override, dict) and pod_override.get(Encoding.TYPE):
-                    value['pod_override'] = BaseSerialization()._deserialize(pod_override)
+                    value['pod_override'] = BaseSerialization.deserialize(pod_override)
             return value
 
         return process
@@ -253,7 +260,7 @@ class Interval(TypeDecorator):
         return data
 
 
-def skip_locked(session: Session) -> Dict[str, Any]:
+def skip_locked(session: Session) -> dict[str, Any]:
     """
     Return kargs for passing to `with_for_update()` suitable for the current DB engine version.
 
@@ -273,7 +280,7 @@ def skip_locked(session: Session) -> Dict[str, Any]:
         return {}
 
 
-def nowait(session: Session) -> Dict[str, Any]:
+def nowait(session: Session) -> dict[str, Any]:
     """
     Return kwargs for passing to `with_for_update()` suitable for the current DB engine version.
 
@@ -293,7 +300,7 @@ def nowait(session: Session) -> Dict[str, Any]:
         return {}
 
 
-def nulls_first(col, session: Session) -> Dict[str, Any]:
+def nulls_first(col, session: Session) -> dict[str, Any]:
     """
     Adds a nullsfirst construct to the column ordering. Currently only Postgres supports it.
     In MySQL & Sqlite NULL values are considered lower than any non-NULL value, therefore, NULL values
@@ -398,7 +405,7 @@ def is_lock_not_available_error(error: OperationalError):
 
 
 def tuple_in_condition(
-    columns: Tuple[ColumnElement, ...],
+    columns: tuple[ColumnElement, ...],
     collection: Iterable[Any],
 ) -> ColumnOperators:
     """Generates a tuple-in-collection operator to use in ``.filter()``.
